const { forwardTo } = require('prisma-binding')
const bcrypt = require('bcryptjs')
const jwt = require('jsonwebtoken')

const LoginError = new Error('*ERR_LOGIN* You must be logged in.')

const Query = {
  users: forwardTo('db'),
  training: async (parent, args, context, info) => {
    if (!context.request.userId) throw LoginError
    return context.db.query.training({ data: args }, info)
  },
  trainings: async (parent, args, context, info) => {
    if (!context.request.userId) throw LoginError
    return context.db.query.trainings({}, info)
  },
  trainingTypes: async (parent, args, context, info) => {
    if (!context.request.userId) throw LoginError
    return context.db.query.trainingTypes()
  },
  blocks: async (parent, args, context, info) => {
    if (!context.request.userId) throw LoginError
    return context.db.query.trainingTypes()
  },
  me: (parent, args, context, info) => {
    if (!context.request.userId) throw LoginError
    return context.db.query.user({ where: { id: context.request.userId } }, info)
  }
}

const Mutation = {
  createUser: async (parent, args, context, info) => {
    const email = args.email.toLowerCase()
    const password = await bcrypt.hash(args.password, 10)
    const user = await context.db.mutation.createUser(
      {
        data: {
          ...args,
          email,
          password
        }
      },
      info
    )
    return user
  },
  signup: async (parent, args, ctx, info) => {
    const email = args.email.toLowerCase()
    const password = await bcrypt.hash(args.password, 10)
    const user = await ctx.db.mutation.createUser(
      {
        data: {
          ...args,
          email,
          password
        }
      },
      info
    )
    const token = jwt.sign({ userId: user.id }, process.env.APP_SECRET)
    ctx.response.cookie('token', token, {
      httpOnly: true,
      maxAge: 24 * 60 * 60 * 1000
    })
    return user
  },
  login: async (parent, args, context, info) => {
    const { email, password } = args
    const user = await context.db.query.user({ where: { email } })
    if (!user) throw new Error('User not found')
    const valid = await bcrypt.compare(password, user.password)
    if (!valid) throw new Error('Invalid password')
    const token = jwt.sign({ userId: user.id }, process.env.APP_SECRET)
    context.response.cookie(
      'token',
      token,
      {
        httpOnly: true,
        maxAge: 7 * 24 * 3600 * 1000
      },
      info
    )
    return user
  },
  logout: async (parent, args, context, info) => {
    console.log('trying to log out')
    context.response.clearCookie('token')
    console.log('cookie cleared.')
    return 'Logged out.'
  },
  createTraining: async (parent, args, context, info) => {
    if (!context.request.userId) throw LoginError
    const training = await context.db.mutation.createTraining(
      { data: args }, info
    )
    return training
  },
  createTrainingType: async (parent, args, context, info) => {
    if (!context.request.userId) throw LoginError
    const trainingType = await context.db.mutation.createTrainingType(
      { data: args }, info
    )
    return trainingType
  },
  createBlock: async (parent, args, context, info) => {
    if (!context.request.userId) throw LoginError
    const block = await context.db.mutation.createBlock(
      { data: args }, info
    )
    return block
  }
}

const resolvers = {
  Query,
  Mutation
}

module.exports = { resolvers }
